/*
 * tulklut_it.c
 *
 * Zorglang transformation library
 *
 * Copyright (C) 2003-2004 Richard "riri_le_mechant" GILL <richard@nasgaia.org>
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc.,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "tulklut.h"

/* tulklut_it()
 * most basic function to transform a string in Zorglang.
 * it checks non alpha characters to split the string in two ones, and
 * do a recursive tulklut_it().
 * the *len* parameter is really important for two things:
 * - we can assure to work on a fixed len of chars, not the 'zeroed' length,
 *   this limit buffer overflow bugs/attack.
 * - the recursive call is easier to code, and run faster, because we can
 *   this way work on a 'frame' of the string.
 * the return value is the 'str' parameter equivalent
 */
char *tulklut_it(char *str, size_t len)
{
	char *pos;

	/* tulkluting a one letter string is a split */
	if (str == NULL || len < 2)
		return str;

	/* check string length */
	if (strlen(str) < len)
		len = strlen(str);

	/* while characters from end are alphas */
	pos = str + len - 1;
	while (pos > str && isalpha(*pos))
		-- pos;

	/* we found one non alpha char */
	if (pos > str)
	{
		/* split this to tulklut on the two parts */
		size_t len1, len2;
		char *pos2;
		len1 = pos - str;
		pos2 = pos + 1;
		len2 = len - len1 - 1;

		tulklut_it(str, len1);	/* first one */
		tulklut_it(pos2, len2);	/* second one */
	}
	else	/* let's tulklut his string */
	{
		int j, i = 0;
		char c;

		/* first, check uppercase letters */
		c = str[len - 1];
		if (tolower(c) != c && tolower(*str) == *str)
		{
			/* swap uppercases */
			*str = toupper(*str);
			str[len - 1] = tolower(str[len - 1]);
		}
		else if (tolower(*str) != *str && tolower(c) == c)
		{
			/* swap uppercases */
			*str = tolower(*str);
			str[len - 1] = toupper(str[len - 1]);
		}

		for (j = len - 1;  j >= 0; j --, i ++)
		{
			if (j < i)
				break;

			/* this could be optimized with an intrinsic swap function */
			c = str[i];
			str[i] = str[j];
			str[j] = c;
		}
	}

	return str;
}

/* tulklut_dup()
 * this one acts like tulklut_it(), but preserve the original string and
 * allocate a new tulkluted one in return value.
 */
char *tulklut_dup(const char *str, size_t len)
{
	char *res;

	res = (char *) malloc((len + 1) * sizeof(char));
	if (res == NULL)
		return NULL;

	memcpy(res, str, len + 1);
	return tulklut_it(res, len);
}
